import { Message, MessageCode } from "../util/message";
import { Result } from "../util/result";
import { RectangularStructure } from "./rectangular_structure";
import { RectangularStructureParams } from "./rectangular_structure_params";
import { Structure, StructureFlowMode, StructureFlowRegime } from "./structure";
import { LoiDebit } from "./structure_props";

/**
 * Equation classique seuil dénoyé
 */
export class StructureWeirFree extends RectangularStructure {

    constructor(prms: RectangularStructureParams, dbg: boolean = false) {
        super(prms, dbg);
        this._loiDebit = LoiDebit.WeirFree;
        this.prms.CdWR.visible = true;
    }

    public Calc(sVarCalc: string, rInit?: number): Result {
        this.currentResult = super.Calc(sVarCalc, rInit);
        // do not check h2 for derived classes (ex: StructureWeirVillemonte)
        if (this._loiDebit === LoiDebit.WeirFree && this.prms.h2.v > 0) {
            this._result.resultElement.addMessage(new Message(
                MessageCode.WARNING_DOWNSTREAM_ELEVATION_POSSIBLE_SUBMERSION,
                { h2: this.prms.h2.v }
            ));
        }
        return this._result;
    }

    /**
     * Calcul analytique Q = f(Cd, L, h1, h2, W) seuil dénoyé
     * @param sVarCalc Variable à calculer (doit être "Q")
     */
    public CalcQ(): Result {
        const data = this.getResultData();

        const v = this.prms.CdWR.v * this.prms.L.v * Structure.R2G * Math.pow(this.prms.h1.v, 1.5);

        return new Result(v, this, data);
    }

    protected getFlowRegime() {
        return StructureFlowRegime.FREE;
    }

    protected getFlowMode() {
        return StructureFlowMode.WEIR;
    }
}
